﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using vloume;
using System.Runtime.InteropServices;
using System.Threading;

namespace ShiJie_GreatWallSDK
{

    public partial class FrmMain : Form
    {
        public class Line
        {
            public Line(Point startPoint)
            {
                StartPoint = startPoint;
                EndPoint = startPoint;
            }
            public Point StartPoint = Point.Empty;
            public Point EndPoint = Point.Empty;
        } 

        //draw rectangle
        private Point start;
        private Boolean blnDraws = false;
        private Rectangle rectCali;
        private Rectangle rectSafe;
        private int[] iPackageInfo = null;
        private double dZoomCoffe = 0.0;
        private bool bInit3D = false;

        private Line drawingLine1 = null;
        private Line drawingLine2 = null;
        private Line drawingLine3 = null;
        private Line drawingLine4 = null;
        
        private object objUpdateUI = new object();
        private List<Line> lines = new List<Line>();

        private Thread volumeThread = null;
        private bool threadRun = false;
        private int iWidth = 640;
        private int iHeight = 480;
        private byte[] ptrImage;
        private int[] iObjectPoint;

        #region 
        private Rectangle bgRect;
        private Rectangle safeRect;
        private IntPtr intPtrbgRect;
        private IntPtr intPtrsafeRect;
        private bool m_bSafePlane = false;
        #endregion

        private int PlaneX = 0;
        private int PlaneY = 0;
        private int PlaneW = 0;
        private int PlaneH = 0;

        private ManualResetEvent syncMeasure = new ManualResetEvent(false);
        private delegate void InvokeDelegate();

        public FrmMain()
        {
            InitializeComponent();
            InitializeEnvironmentVariable();
        }

        private void InitializeEnvironmentVariable()
        {
            string pathvar = System.Environment.GetEnvironmentVariable("PATH");

            if (true == pathvar.EndsWith(";"))
            {
                System.Environment.SetEnvironmentVariable("PATH", pathvar + AppDomain.CurrentDomain.BaseDirectory + @"\ShiJieSDK");
            }
            else
            {
                System.Environment.SetEnvironmentVariable("PATH", pathvar + ";" + AppDomain.CurrentDomain.BaseDirectory + @"\ShiJieSDK");
            }
        }

        private void FrmMain_Load(object sender, EventArgs e)
        {
            InitVolumeSystem();
            this.syncMeasure.Reset();
        }

        private void InitVolumeSystem()
        {
            int err = -1;
            err = CTyvloume.TYVolumeSystemInit(true, false);
            if (err == 0)
            {
                PlaneX = 120;
                PlaneY = 90;
                PlaneW = 400;
                PlaneH = 300;
                initVolumeParameters();
                setSafeAndCalibrationPlane();
                markSafeAndCalibrationPlane();
                initMeasureVolumeThread();
                bInit3D = true;
            }
        }

        private void initVolumeParameters()
        {
            int err;

            IntPtr paraPtr = Marshal.AllocHGlobal(sizeof(int));

            try
            {
                Marshal.WriteInt32(paraPtr, 0);
                err = CTyvloume.TYVolumeSystemWriteProperty(TY_PM_TYPE.ID_PM_bool_ENABLE_COLOR, paraPtr, sizeof(int));

                Marshal.WriteInt32(paraPtr, 20);
                err = CTyvloume.TYVolumeSystemWriteProperty(TY_PM_TYPE.ID_PM_int_THIN_THRED, paraPtr, sizeof(int));

                Marshal.WriteInt32(paraPtr, 0);
                err = CTyvloume.TYVolumeSystemWriteProperty(TY_PM_TYPE.ID_PM_bool_ALWAYS_BOX, paraPtr, sizeof(int));

                Marshal.WriteInt32(paraPtr, 0);
                err = CTyvloume.TYVolumeSystemWriteProperty(TY_PM_TYPE.ID_PM_bool_REDO_UNBLANK_IRREG, paraPtr, sizeof(int));

                Marshal.WriteInt32(paraPtr, 20);
                err = CTyvloume.TYVolumeSystemWriteProperty(TY_PM_TYPE.ID_PM_int_LEAST_HEIGHT, paraPtr, sizeof(int));

                Marshal.WriteInt32(paraPtr, 1);
                err = CTyvloume.TYVolumeSystemWriteProperty(TY_PM_TYPE.ID_PM_bool_ONLY_SAFERECT, paraPtr, sizeof(int));

                Marshal.FreeHGlobal(paraPtr);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }

        }

        private void initMeasureVolumeThread()
        {
            this.ptrImage = new byte[iWidth * iHeight];
            this.iObjectPoint = new int[8];
            this.iPackageInfo = new int[3];
            this.volumeThread = new Thread(this.measureVolumeThread);
            this.threadRun = true;
            this.volumeThread.Start();
        }

        private void markSafeAndCalibrationPlane()
        {
            blnDraws = true;

            dZoomCoffe = (double)pbVolumeView.Width / iWidth;

            rectCali.X = (int)((double)PlaneX * dZoomCoffe);
            rectCali.Y = (int)((double)PlaneY * dZoomCoffe);
            rectCali.Width = (int)((double)PlaneW * dZoomCoffe);
            rectCali.Height = (int)((double)PlaneH * dZoomCoffe);

            rectSafe.X = (int)((double)70 * dZoomCoffe);
            rectSafe.Y = (int)((double)40 * dZoomCoffe);
            rectSafe.Width = (int)((double)500 * dZoomCoffe);
            rectSafe.Height = (int)((double)400 * dZoomCoffe);
            
        }

        private void setSafeAndCalibrationPlane()
        {
            int err;

            bgRect = new Rectangle(PlaneX, PlaneY, PlaneW, PlaneH);
            safeRect = new Rectangle(70, 40, 500, 400);

            int nSizeOfbgRect = Marshal.SizeOf(bgRect);
            intPtrbgRect = Marshal.AllocHGlobal(nSizeOfbgRect);
            Marshal.StructureToPtr(bgRect, intPtrbgRect, true);

            int nSizeOfsafeRect = Marshal.SizeOf(safeRect);
            intPtrsafeRect = Marshal.AllocHGlobal(nSizeOfsafeRect);
            Marshal.StructureToPtr(safeRect, intPtrsafeRect, true);

            // set bg rect
            err = CTyvloume.TYVolumeSystemWriteProperty(TY_PM_TYPE.ID_PM_int4_DEPTH_ROI, intPtrbgRect, nSizeOfbgRect);
            // set safe rect
            err = CTyvloume.TYVolumeSystemWriteProperty(TY_PM_TYPE.ID_PM_int4_SAFE_RECT, intPtrsafeRect, nSizeOfsafeRect);

            Marshal.FreeHGlobal(intPtrbgRect);
            Marshal.FreeHGlobal(intPtrsafeRect);
        }

        private void pbVolumeView_Paint(object sender, PaintEventArgs e)
        {
            if (blnDraws)
            {
                if (pbVolumeView.Image != null)
                {
                    lock (pbVolumeView.Image)
                    {
                        if (rectCali != null && rectCali.Width > 0 && rectCali.Height > 0)
                        {
                            e.Graphics.DrawRectangle(new Pen(Color.Green, 3), rectCali);
                        }

                        if (rectSafe != null && rectSafe.Width > 0 && rectSafe.Height > 0)
                        {
                            e.Graphics.DrawRectangle(new Pen(Color.Red, 3), rectSafe);
                        }
                    }
                }
            }

            lock (objUpdateUI)
            {
                foreach (Line line in lines)
                {
                    e.Graphics.DrawLine(new Pen(Color.Blue, 2), line.StartPoint, line.EndPoint);
                }
            }
        }

        private void pbVolumeView_MouseDown(object sender, MouseEventArgs e)
        {
            start = e.Location;
            Invalidate();
            blnDraws = true;
        }

        private void pbVolumeView_MouseMove(object sender, MouseEventArgs e)
        {
            if (blnDraws)
            {
                if (e.Button != MouseButtons.Left)//判断是否按下左键
                    return;
                Point tempEndPoint = e.Location; //记录框的位置和大小
                rectCali.Location = new Point(
                Math.Min(start.X, tempEndPoint.X),
                Math.Min(start.Y, tempEndPoint.Y));
                rectCali.Size = new Size(
                Math.Abs(start.X - tempEndPoint.X),
                Math.Abs(start.Y - tempEndPoint.Y));
                pbVolumeView.Invalidate();
            }  
        }

        private void ConvertDepthFrame(short[] depthFrame, byte[] depthFrame32, Int32 iWidth, Int32 iHeight)
        {
            int depthLen = iWidth * iHeight;
            int colorLen = iWidth * iHeight * 4;

            //Run through the depth frame making the correlation between the two arrays
            for (int i16 = 0, i32 = 0; i16 < depthLen && i32 < colorLen; i16++, i32 += 4)
            {

                int realDepth = depthFrame[i16] >> 3;

                depthFrame32[i32 + 0] = (byte)(realDepth);
                depthFrame32[i32 + 1] = (byte)(realDepth);
                depthFrame32[i32 + 2] = (byte)(realDepth);
            }
        }

        private void markPackagePosition()
        {
            Point StartPoint = Point.Empty;
            Point EndPoint = Point.Empty;

            lock (objUpdateUI)
            {
                lines.Clear();
                
                dZoomCoffe = (double)pbVolumeView.Width / iWidth;
               

                StartPoint.X = (int)((double)iObjectPoint[0] * dZoomCoffe);
                StartPoint.Y = (int)((double)iObjectPoint[1] * dZoomCoffe);
                EndPoint.X = (int)((double)iObjectPoint[2] * dZoomCoffe);
                EndPoint.Y = (int)((double)iObjectPoint[3] * dZoomCoffe);

                drawingLine1 = new Line(StartPoint);
                drawingLine1.EndPoint = EndPoint;
                lines.Add(drawingLine1);

                StartPoint.X = (int)((double)iObjectPoint[2] * dZoomCoffe);
                StartPoint.Y = (int)((double)iObjectPoint[3] * dZoomCoffe);
                EndPoint.X = (int)((double)iObjectPoint[4] * dZoomCoffe);
                EndPoint.Y = (int)((double)iObjectPoint[5] * dZoomCoffe);
                drawingLine2 = new Line(StartPoint);
                drawingLine2.EndPoint = EndPoint;
                lines.Add(drawingLine2);

                StartPoint.X = (int)((double)iObjectPoint[4] * dZoomCoffe);
                StartPoint.Y = (int)((double)iObjectPoint[5] * dZoomCoffe);
                EndPoint.X = (int)((double)iObjectPoint[6] * dZoomCoffe);
                EndPoint.Y = (int)((double)iObjectPoint[7] * dZoomCoffe);
                drawingLine3 = new Line(StartPoint);
                drawingLine3.EndPoint = EndPoint;
                lines.Add(drawingLine3);

                StartPoint.X = (int)((double)iObjectPoint[0] * dZoomCoffe);
                StartPoint.Y = (int)((double)iObjectPoint[1] * dZoomCoffe);
                EndPoint.X = (int)((double)iObjectPoint[6] * dZoomCoffe);
                EndPoint.Y = (int)((double)iObjectPoint[7] * dZoomCoffe);
                drawingLine4 = new Line(StartPoint);
                drawingLine4.EndPoint = EndPoint;
                lines.Add(drawingLine4);
            }

        }

        private void measurePackageVolume()
        {
            // Don't do anything if the form's handle hasn't been created 
            // or the form has been disposed.
            if (!this.IsHandleCreated && !this.IsDisposed) return;

            // Invoke an anonymous method on the thread of the form.
            this.InvokeOnUiThreadIfRequired(() => lbShowLength.Text = "--");
            this.InvokeOnUiThreadIfRequired(() => lbWidth.Text = "--");
            this.InvokeOnUiThreadIfRequired(() => lbHeight.Text = "--");
            this.InvokeOnUiThreadIfRequired(() => lbShowVolume.Text = "--");

            lock (objUpdateUI)
            {
                lines.Clear();
            }           
      
            TYPackageInfo result = new TYPackageInfo();
            int err = CTyvloume.TYVolumeSystemGetVolume(ref result);
            if (0 == err)
            {
                iPackageInfo[0] = (int)result.sizeX;
                iPackageInfo[1] = (int)result.sizeY;
                iPackageInfo[2] = (int)result.sizeZ;

                iObjectPoint[0] = (int)result.pixelPoints[0];
                iObjectPoint[1] = (int)result.pixelPoints[1];
                iObjectPoint[2] = (int)result.pixelPoints[2];
                iObjectPoint[3] = (int)result.pixelPoints[3];
                iObjectPoint[4] = (int)result.pixelPoints[4];
                iObjectPoint[5] = (int)result.pixelPoints[5];
                iObjectPoint[6] = (int)result.pixelPoints[6];
                iObjectPoint[7] = (int)result.pixelPoints[7];

                this.InvokeOnUiThreadIfRequired(() => lbShowLength.Text = iPackageInfo[0].ToString());
                this.InvokeOnUiThreadIfRequired(() => lbWidth.Text = iPackageInfo[1].ToString());
                this.InvokeOnUiThreadIfRequired(() => lbHeight.Text = iPackageInfo[2].ToString());
                this.InvokeOnUiThreadIfRequired(() => lbShowVolume.Text = (iPackageInfo[0] * iPackageInfo[1] * iPackageInfo[2]).ToString());
                markPackagePosition();
            }
            else
            {
                iPackageInfo[0] = 0;
                iPackageInfo[1] = 0;
                iPackageInfo[2] = 0;
            }
        }

        private void measureVolumeThread()
        {
            int status = 0;
            TY_FRAME_DATA frameData = new TY_FRAME_DATA();
            IntPtr ptrFrmaeData = IntPtr.Zero;
          
            while (this.threadRun)
            {
                status = CTyvloume.TYVolumeSystemGetImage(ref frameData, 100);
                if (status != 0)
                {
                    Marshal.FreeHGlobal(ptrFrmaeData);
                    continue;
                }   

                Bitmap bmap = null;

                for (int i = 0; i < frameData.validCount; i++)
                {
                    TY_IMAGE_DATA image = frameData.image[i];

                    if (image.componentID == 0x00010000)
                    {
                        byte[] depthFrame32 = new byte[frameData.image[i].width * frameData.image[i].height * 4];
                        short[] depthFrame16 = new short[frameData.image[i].width * frameData.image[i].height];

                        Marshal.Copy(frameData.image[i].buffer, depthFrame16, 0, depthFrame16.Length);

                        ConvertDepthFrame(depthFrame16, depthFrame32, frameData.image[i].width, frameData.image[i].height);

                        bmap = new Bitmap(frameData.image[i].width, frameData.image[i].height, PixelFormat.Format32bppRgb);
                        BitmapData bmapdata = bmap.LockBits(new Rectangle(0, 0, frameData.image[i].width, frameData.image[i].height), ImageLockMode.WriteOnly, bmap.PixelFormat);
                        IntPtr ptr = bmapdata.Scan0;
                        Marshal.Copy(depthFrame32, 0, ptr, 4 * frameData.image[i].width * frameData.image[i].height);
                        bmap.UnlockBits(bmapdata);
                    }
                    else
                    {
                        continue;
                    }

                    this.InvokeOnUiThreadIfRequired(() => pbVolumeView.Image = bmap);
                    if (syncMeasure.WaitOne(50))
                    {
                        syncMeasure.Reset();
                        this.Invoke(new InvokeDelegate(measurePackageVolume));
                    }
               }
                Marshal.FreeHGlobal(ptrFrmaeData);

                if (true == m_bSafePlane)
                {
                    setSafeAndCalibrationPlane();
                    try
                    {
                        this.Invoke((EventHandler)delegate
                        {
                            int err = CTyvloume.TYVolumeSystemCalib();

                            if (0 != err)
                            {
                                MessageBox.Show(ShiJie_GreatWallSDK.Properties.Resources.CALIBRATION_FAILED, ShiJie_GreatWallSDK.Properties.Resources.POPUP_HINT);
                            }
                        });
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message);
                    }

                    m_bSafePlane = false;
                    tsbCalibration.Enabled = true;
                }

                if (syncMeasure.WaitOne(50))
                {
                    syncMeasure.Reset();
                    this.Invoke(new InvokeDelegate(measurePackageVolume));
                }

                Thread.Sleep(100);
                GC.Collect();
            }

        }

        private void tsbCalibration_Click(object sender, EventArgs e)
        {
            Rectangle selectRect = new Rectangle();
 
            if (false == bInit3D)
            {
                MessageBox.Show(ShiJie_GreatWallSDK.Properties.Resources.VOLUME_UNREADY, ShiJie_GreatWallSDK.Properties.Resources.POPUP_HINT);
                return;
            }

            tsbCalibration.Enabled = false;
            DialogResult dr = MessageBox.Show(ShiJie_GreatWallSDK.Properties.Resources.CALIBRATION_HINT, ShiJie_GreatWallSDK.Properties.Resources.POPUP_HINT,
                              MessageBoxButtons.YesNo);

            if (dr == DialogResult.Yes)
            {
                dZoomCoffe = (double)iWidth / pbVolumeView.Width;

                selectRect.X = (int)((double)rectCali.X * dZoomCoffe);
                selectRect.Y = (int)((double)rectCali.Y * dZoomCoffe);
                selectRect.Width = (int)((double)rectCali.Width * dZoomCoffe);
                selectRect.Height = (int)((double)rectCali.Height * dZoomCoffe);

                PlaneX = selectRect.X;
                PlaneY = selectRect.Y;
                PlaneW = selectRect.Width;
                PlaneH = selectRect.Height;

                m_bSafePlane = true;
                MessageBox.Show(ShiJie_GreatWallSDK.Properties.Resources.CALIBRATION_SUCESS, ShiJie_GreatWallSDK.Properties.Resources.POPUP_HINT);
            }
            tsbCalibration.Enabled = true;
        }

        private void FrmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (volumeThread != null)
            {
                this.threadRun = false;
                if (volumeThread != null && volumeThread.IsAlive)
                {
                    volumeThread.Abort();
                }
                volumeThread.Join();
                this.volumeThread = null;
            }

            try
            {
                if (true == bInit3D)
                {
                    CTyvloume.TYVolumeSystemDeinit();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, ShiJie_GreatWallSDK.Properties.Resources.POPUP_HINT);
            }
        }

        private void tsbVolumeTrigger_Click(object sender, EventArgs e)
        {
            if (false == bInit3D)
            {
                MessageBox.Show(ShiJie_GreatWallSDK.Properties.Resources.VOLUME_UNREADY, ShiJie_GreatWallSDK.Properties.Resources.POPUP_HINT);
                return;
            }
            syncMeasure.Set();
        }       
    }
}
