﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;

namespace vloume
{
    public enum TY_PM_TYPE{
        ID_PM_int_MIN_AREA = 0x30040101,
        ID_PM_int_BG_CNT = 0x30040102,
        ID_PM_int4_DEPTH_ROI = 0x30040103,
        ID_PM_int4_COLOR_ROI = 0x30040104,
        ID_PM_int_LEAST_HEIGHT = 0x30040105,
        ID_PM_int_PIX_CUT_FOR_PLANE = 0x30040106,
        ID_PM_bool_OPTIMIZE_BOX = 0x30040107,
        ID_PM_int2_BOX_CUTTING_UPDOWN = 0x30040108,
        ID_PM_float_MAX_PLANE_SD = 0x30040109,
        ID_PM_float_EDGE_CUTTING_COEF = 0x3004010a,
        ID_PM_float_MAX_SHRINK_COEF = 0x3004010b,
        ID_PM_int4_SAFE_RECT = 0x3004010c,
        ID_PM_bool_MASK_BLACK = 0x3004010d,
        ID_PM_float3_BOX_FIX = 0x3004010e,
        ID_PM_float3_IRREG_FIX = 0x3004010f,
        ID_PM_float3n_XY_FIX_BY_Z = 0x30040110,
        ID_PM_bool_USE_MOVE_FIX = 0x30040111,
        ID_PM_float2_MOVE_FIX = 0x30040112,
        ID_PM_bool_USE_SPEED_FIX = 0x30040113,
        ID_PM_float2_SPEED_FIX = 0x30040114,
        ID_PM_float_ROTATE_FIX = 0x30040115,
        ID_PM_float_BASE_PLANE_OFFSET = 0x30040116,
        ID_PM_bool_ONLY_SAFERECT = 0x30040117,
        ID_PM_bool_MEASURE_TOTAL = 0x30040118,
        ID_PM_int_PROJECTED_BG_Z = 0x30040119,
        ID_PM_bool_REDO_UNBLANK_IRREG = 0x3004011a,
        ID_PM_bool_NEVER_BOX = 0x3004011b,
        ID_PM_bool_THIN_OBJ = 0x3004011e,
        ID_PM_int_THIN_THRED = 0x30040120,
        ID_PM_bool_THIN_OBJ_AUTO = 0x30040121,
        ID_PM_bool_ALWAYS_BOX = 0x30040123,
        ID_PM_bool_ENABLE_COLOR = 0x20010104,
    }
   
    //------------------------------------------------------------------------------
    //  Buffer & Callback
    //------------------------------------------------------------------------------
    public  struct TY_IMAGE_DATA
    {
        public ulong timestamp;             ///< Timestamp in microseconds
        public int imageIndex;             ///< image index used in trigger mode
        public int status;                 ///< Status of this buffer
        public int componentID;            ///< Where current data come from
        public int size;                   ///< Buffer size
        public IntPtr buffer;                 ///< Pointer to data buffer
        public int width;                  ///< Image width in pixels
        public int height;                 ///< Image height in pixels
        public int pixelFormat;            ///< Pixel format, see TY_PIXEL_FORMAT_LIST
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
        public int[] reserved;            ///< Reserved
    };


    public struct TY_FRAME_DATA
    {
        public IntPtr userBuffer;     ///< Pointer to user enqueued buffer, user should enqueue this buffer in the end of callback
        public int bufferSize;     ///< Size of userBuffer
        public int validCount;     ///< Number of valid data
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 6)]
        public int[] reserved;    ///< Reserved
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 10)]
        public TY_IMAGE_DATA[] image;      ///< Buffer data, max to 10 images per frame, each buffer data could be an image or something else.
    };



    public struct TYPackageInfo
    {
        public char type;           ///< 0:no package, 1:box, 2:irregular
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
        public char[] __rsvd0;
        public float sizeX;
        public float sizeY;
        public float sizeZ;
        public float boundingVolume; ///< accumulated vol
        public float integeralVolume;///< accumulated vol
        public float centerPosX;
        public float centerPosY;
        public float posAngle;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
        public int[] __rsvd1;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
        public float[] pixelPoints; ///< vertex of bounding box on depth image, {x,y}[4]
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
        public float[] pixelPointsRGB; ///< vertex of bounding box on color image, {x,y}[4]
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct CvRect
    {
        [MarshalAs(UnmanagedType.I4)]
        public int x; /* 方形的左上角的x-坐标 */
        [MarshalAs(UnmanagedType.I4)]
        public int y; /* 方形的左上角的y-坐标*/
        [MarshalAs(UnmanagedType.I4)]
        public int width; /* 宽 */
        [MarshalAs(UnmanagedType.I4)]
        public int height; /* 高 */
    }

    public class CTyvloume
    {

        [DllImport("PackageMeasure.dll", EntryPoint = "TYVolumeSystemInit")]
        public static extern int TYVolumeSystemInit(bool grabDepthImage, bool grabColorImage);

        [DllImport("PackageMeasure.dll", EntryPoint = "TYVolumeSystemDeinit")]
        public static extern int TYVolumeSystemDeinit();

        [DllImport("PackageMeasure.dll", EntryPoint = "TYVolumeSystemGetImage")]
        public static extern int TYVolumeSystemGetImage(ref TY_FRAME_DATA out_frame, int timeout_ms);

        [DllImport("PackageMeasure.dll", EntryPoint = "TYVolumeSystemCalib")]
        public static extern int TYVolumeSystemCalib();

        [DllImport("PackageMeasure.dll", EntryPoint = "TYVolumeSystemGetVolume")]
        public static extern int TYVolumeSystemGetVolume(ref TYPackageInfo result);

        [DllImport("PackageMeasure.dll", EntryPoint = "TYVolumeSystemReadProperty")]
        public static extern int TYVolumeSystemReadProperty(TY_PM_TYPE prop_id,IntPtr buf, int buflen);

        [DllImport("PackageMeasure.dll", EntryPoint = "TYVolumeSystemWriteProperty")]
        public static extern int TYVolumeSystemWriteProperty(TY_PM_TYPE prop_id, IntPtr buf, int buflen);
    }
}
